@@ -50,11 +50,8 @@ use super::{bai, Record, MAGIC_NUMBER};
 ///
 /// # Ok::<(), io::Error>(())
 /// ```
-pub struct Reader<R>
-where
-    R: Read,
-{
-    inner: bgzf::Reader<R>,
+pub struct Reader<R> {
+    inner: R,
     buf: Vec<u8>,
 }
 
@@ -62,22 +59,46 @@ impl<R> Reader<R>
 where
     R: Read,
 {
-    /// Creates a BAM reader.
-    ///
-    /// The given reader must be a raw BGZF stream, as the underlying reader wraps it in a decoder.
+    /// Returns a reference to the underlying reader.
     ///
     /// # Examples
     ///
     /// ```
     /// use noodles_bam as bam;
     /// let data = [];
-    /// let reader = bam::Reader::new(&data[..]);
+    /// let reader = bam::Reader::from(&data[..]);
+    /// assert!(reader.get_ref().is_empty());
     /// ```
-    pub fn new(reader: R) -> Self {
-        Self {
-            inner: bgzf::Reader::new(reader),
-            buf: Vec::new(),
-        }
+    pub fn get_ref(&self) -> &R {
+        &self.inner
+    }
+
+    /// Returns a mutable reference to the underlying reader.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use noodles_bam as bam;
+    /// let data = [];
+    /// let mut reader = bam::Reader::from(&data[..]);
+    /// assert!(reader.get_mut().is_empty());
+    /// ```
+    pub fn get_mut(&mut self) -> &mut R {
+        &mut self.inner
+    }
+
+    /// Returns the underlying reader.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use noodles_bam as bam;
+    /// let data = [];
+    /// let reader = bam::Reader::from(&data[..]);
+    /// assert!(reader.into_inner().is_empty());
+    /// ```
+    pub fn into_inner(self) -> R {
+        self.inner
     }
 
     /// Reads the raw SAM header.
@@ -186,6 +207,26 @@ where
     pub fn records(&mut self) -> Records<'_, R> {
         Records::new(self)
     }
+}
+
+impl<R> Reader<bgzf::Reader<R>>
+where
+    R: Read,
+{
+    /// Creates a BAM reader.
+    ///
+    /// The given reader must be a raw BGZF stream, as the underlying reader wraps it in a decoder.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use noodles_bam as bam;
+    /// let data = [];
+    /// let reader = bam::Reader::new(&data[..]);
+    /// ```
+    pub fn new(reader: R) -> Self {
+        Self::from(bgzf::Reader::new(reader))
+    }
 
     /// Returns the current virtual position of the underlying BGZF reader.
     ///
@@ -208,7 +249,7 @@ where
     }
 }
 
-impl<R> Reader<R>
+impl<R> Reader<bgzf::Reader<R>>
 where
     R: Read + Seek,
 {
@@ -308,6 +349,15 @@ where
     }
 }
 
+impl<R> From<R> for Reader<R> {
+    fn from(inner: R) -> Self {
+        Self {
+            inner,
+            buf: Vec::new(),
+        }
+    }
+}
+
 fn read_magic<R>(reader: &mut R) -> io::Result<()>
 where
     R: Read,
